/**
 * @Author：杨振忠
 * @Date 21/06/19
 * @Version 1.0
 * @Description request封装
 */

import config from '../config.js';

let getTokenInfoIng = false;
let requestNum = 0,
	loadingtimer;

/**
 * 设置当前request数量
 * @param isAdd 是否添加一条
 */
const setRequestNum = (isAdd) => {
	if (isAdd) {
		requestNum++;
	} else {
		requestNum--;
		clearTimeout(loadingtimer);
		loadingtimer = setTimeout(() => {
			if (requestNum <= 0) {
				uni.hideLoading()
			}
		}, 300)
	}
}

/**
 * 发起请求 async
 * @param method 访问方法
 * @param url 地址
 * @param data 访问数据
 * @param header 请求头
 * @param withToken 是否携带token
 */
const request = async (method, url, data, header, withToken = null) => {
	const params = {};

	if (!url) {
		throw new Error("URL is Empty");
	}

	if (data) {
		//自动判断url参数
		let urlParams = url.match(/\{([^>]+?)\}/g);
		urlParams && urlParams.forEach(params => {
			let realkeyStr = params.substr(1, params.length - 2);
			if (data && data.hasOwnProperty(realkeyStr)) {
				url = url.replace(params, data[realkeyStr]);
				delete data[realkeyStr];
			}
		})
		if (JSON.stringify(data) == "{}") {
			data = null;
		}
	}

	params.url = url;
	if (url.indexOf("http://") == -1 && url.indexOf("https://") == -1) {
		if (!config.request.url) {
			throw new Error("Domain is Empty!");
		}
		params.url = config.request.url + url;
	}
	method && (params.method = method);
	data && (params.data = data);
	header && (params.header = header);
	(withToken == undefined || withToken == null) && (withToken = config.request.withToken);

	if (withToken) {
		let tokenInfo = uni.getStorageSync('tokenInfo');
		if (!tokenInfo) {
			config.jwt.authFail(403);
		}
		if (tokenInfo && tokenInfo.hasOwnProperty('expires_in') && tokenInfo.hasOwnProperty('expires_time')) {
			let nowTime = Date.parse(new Date()) / 1000;
			if (tokenInfo.expires_time <= nowTime) {
				await getTokenInfo(tokenInfo);
				tokenInfo = uni.getStorageSync('tokenInfo');
			}
			(!params.header) && (params.header = {});
			params.header['Authorization'] = `Bearer ${tokenInfo.access_token}`;
		}

	}

	return new Promise((resolve, reject) => {
		if (config.request.showLoading) {
			uni.showLoading({
				mask: true,
				title: config.request.showLoading
			})
			setRequestNum(true);
		}
		uni.request(params).then((res) => {
			const resData = res
			if (resData.statusCode === 401 || resData.statusCode === 403) {
				config.jwt.authFail(resData.statusCode);
				return false;
			}
			if (config.request.doneHook) {
				config.request.doneHook(resData, resolve, reject);
			} else {
				resolve(resData);
			}
			setRequestNum();
		}).catch(res => {
			reject(res);
			setRequestNum();
		})
	});
};


/**
 * 获取Token
 */
const getTokenInfo = (tokenInfo) => {
	return new Promise((resolve, reject) => {
		if (getTokenInfoIng) {
			uni.$once("__$getTokenInfo_$complete__", () => {
				resolve()
			})
		} else {
			getTokenInfoIng = true;
			const data = config.jwt.data;
			data.refresh_token = tokenInfo.refresh_token;
			uni.request({
				url: config.request.url + config.jwt.url,
				method: 'POST',
				header: config.jwt.header,
				data: data
			}).then(([msg, res]) => {
				if (res.statusCode === 401 || res.statusCode === 403) {
					config.jwt.authFail(res.statusCode);
				} else if (res.statusCode == 200 && res.data.code == 0) {
					tokenInfo = res.data.data;
					let nowTime = Date.parse(new Date()) / 1000;
					tokenInfo.expires_time = nowTime + Number(tokenInfo.expires_in) - 300;
					uni.setStorageSync("tokenInfo", tokenInfo);
					uni.$emit('__$getTokenInfo_$complete__');
					resolve();
				} else {
					reject();
				}
				getTokenInfoIng = false;
			}).catch(res => {
				reject();
			})
		}

	})
}

export default request;
